import { WorkerCMD, ZmqCMD, } from '@/utils/zmq'
import type { ManualAction, PublishMsg, PubMsgData, SubMsgData, TimeoutMsg, ZmqMessage } from '@/utils/zmq'
import webWorker from '@/utils/zmqJsonWorker?worker'
const env = import.meta.env
const defaultHost = import.meta.env.PROD ? window.location.hostname : '192.168.1.199'

class ZMQJsonWorker {
  private static instance: ZMQJsonWorker | null = null; // ➤ 单例实例
  private worker: Worker;
  private scribeHandlers: Map<string, (msg: SubMsgData | PubMsgData) => void> = new Map();
  private pubTimeoutHandlers: Map<string, (msg: TimeoutMsg) => void> = new Map();
  private host: string;
  private statusCallback: ((status: string) => void) | null = null;
  private isAlwaysListenMsgMap: Map<string, PublishMsg<any>> = new Map();

  private constructor(host: string = defaultHost) {
    this.host = host;
    this.worker = new webWorker();
    this.worker.onmessage = this.handleMessage.bind(this);
  }

  public static getInstance(host: string = defaultHost): ZMQJsonWorker {
    if (!ZMQJsonWorker.instance) {
      ZMQJsonWorker.instance = new ZMQJsonWorker(host);
    }
    return ZMQJsonWorker.instance;
  }

  start() {
    this.worker.postMessage({ cmd: WorkerCMD.START, msg: this.host });
  }

  stop() {
    this.worker.postMessage({ cmd: WorkerCMD.STOP });
    this.worker.terminate();
    ZMQJsonWorker.instance = null; // ➤ 释放实例，允许重新创建
  }

  subscribe(topic: string, handler: (msg: any) => void, id?: string) {
    this.scribeHandlers.set(`${topic}${id ? `-${id}` : ''}`, handler);
    this.worker.postMessage({ cmd: WorkerCMD.SUBSCRIBE, topic });
  }

  unsubscribe(topic: string) {
    // 遍历所有订阅消息，删除包含该主题的订阅
    for (const key in this.scribeHandlers) {
      if (key.startsWith(topic)) {
        this.scribeHandlers.delete(key);
      }
    }

    this.worker.postMessage({ cmd: WorkerCMD.UNSUBSCRIBE, topic });
  }

  publish<T extends ManualAction>(topic: string, msg: PublishMsg<T>, isTimeout: boolean = false, handler?: (msg: TimeoutMsg) => void, isAlwaysListen: boolean = false) {
    if (isTimeout) {
      const timeoutId = msg.id
      if (typeof handler !== 'function') {
        console.warn(`发布主题${topic}失败, 回调函数handler为空`)
        return
      }
      this.pubTimeoutHandlers.set(timeoutId, handler)
    }
    if (isAlwaysListen) {
      this.isAlwaysListenMsgMap.set(`${msg.id}`, msg)
    }
    this.worker.postMessage({ cmd: WorkerCMD.PUBLISH, topic, msg: JSON.stringify(msg), isTimeout, isAlwaysListen });
  }

  setStatusCallback(callback: (status: string) => void) {
    this.statusCallback = callback;
  }

  private handleSubscribeMessage(topic: string, json: PubMsgData & SubMsgData) {
    const handler = this.scribeHandlers.get(topic);
    if (handler) {
      try {
        handler(json);
      } catch (error) {
        console.error(`主题: ${topic} 处理失败:`, error);
      }
    }
  }

  private handleTimeoutMessage(timeoutTopic: string, timeoutId: string) {
    const handler = this.pubTimeoutHandlers.get(timeoutId);
    if (handler) {
      handler({
        timeoutId,
        timeoutTopic
      });
    }
  }

  // 回收发布后订阅的回调
  private GC_pubReleaseSub(key: string) {
    this.scribeHandlers.delete(key);
  }

  // 回收超时消息的回调
  private GC_pubReleaseTimeout(key: string) {
    this.pubTimeoutHandlers.delete(key);
  }

  private handleMessage(e: MessageEvent<ZmqMessage>) {
    const { cmd, msg, topic, community } = e.data;

    if (cmd === ZmqCMD.STATUS) {
      const status = community ? 'disconnected' : 'connected';
      if (this.statusCallback) {
        this.statusCallback(status);
      }
    } else if (cmd === ZmqCMD.JSON_MSG) {
      const json = JSON.parse(msg) as PubMsgData & SubMsgData;

      if (json.action) {
        // 处理订阅消息
        this.handleSubscribeMessage(topic, json);
      } else {
        // 处理需要发布消息并有返回的订阅消息
        this.handleSubscribeMessage(`${topic}-${json.id}`, json);
        // 删除发布消息的回调
        if (Object.keys(json).includes('result') && json.result !== 'progress') {
          if (!this.isAlwaysListenMsgMap.has(`${json.id}`)) {
            this.GC_pubReleaseSub(`${topic}-${json.id}`)
            this.GC_pubReleaseTimeout(`${json.id}`)
          }
        }
      }

    } else if (cmd === ZmqCMD.TIMEOUT) {
      this.handleTimeoutMessage(topic, msg);
      this.GC_pubReleaseTimeout(msg)
      // console.log('pubTimeoutHandlers=>', this.pubTimeoutHandlers)

    }
  }
}

export default ZMQJsonWorker;

