import { ref, computed } from 'vue'
import { defineStore } from 'pinia'
import type { IOnlineDevice, IUpFirmwareStatus } from '@/views/stationData/type'
import ZMQWorker from '@/composables/useZMQJsonWorker'
import { getSubTopic, type SubMsgData } from '@/utils/zmq'
import { getDeviceTopic } from '@/views/stationData/utils'
export const useTransferDataStore = defineStore('transfer', () => {
  const subDevices = getSubTopic('client', 'status', 'transfer')
  const worker = ZMQWorker.getInstance()
  const isConnected = ref(false)

  const connectSite = ref<any>(null)
  async function initConnectSite() {
    if (connectSite.value) return
    connectSite.value = {
      title: '未命名站点',
      edit: true,
      editTitle: '未命名站点',
    }
  }

  const devicesMap = reactive(new Map<string, IOnlineDevice>())
  const checkDeviceStatusInterval = ref<NodeJS.Timeout>()
  function checkDeviceStatus() {
    checkDeviceStatusInterval.value = setInterval(() => {
      const now = Date.now();
      devicesMap.forEach((device: IOnlineDevice, sn) => {
        if (now - device.lastUpdated > 5000) {
          device.status = '离线';
        }
      });
    }, 1000);
  }

  function formatSizeFromKB(num: number): string {
    const sizeKB = Number(num)
    const units = ['KB', 'MB', 'GB', 'TB', 'PB']
    let size = sizeKB
    let unitIndex = 0

    while (size >= 1024 && unitIndex < units.length - 1) {
      size = size / 1024
      unitIndex++
    }

    return `${size.toFixed(2)} ${units[unitIndex]}`
  }

  function getSubDevicesCb(msg: SubMsgData) {
    const { feedback } = msg
    const sn = feedback[1]
    const hasDevice = devicesMap.get(sn)
    if (hasDevice) {
      hasDevice.lastUpdated = Date.now()
      hasDevice.status = '在线'
      hasDevice.footprint = formatSizeFromKB(Number(feedback[4] || 0))
    } else {
      const num = feedback[4] || 0
      const device: IOnlineDevice = {
        clientIp: feedback[0],
        sn: sn,
        site_id: feedback[2],
        versions: feedback[3] ?? '--',
        footprint: formatSizeFromKB(Number(num)),
        lastUpdated: Date.now(),
        status: '在线', // 初始状态为在线
        isChecked: false,
      }
      devicesMap.set(sn, device)
    }

    isConnected.value = devicesMap.size > 0
  }

  const onlineCount = computed(() => {
    return Array.from(devicesMap.values()).filter(item => item.status === '在线')
      .length
  })

  const offlineCount = computed(() => {
    return Array.from(devicesMap.values()).filter(item => item.status === '离线')
      .length
  })

  onMounted(() => {
    worker.subscribe(getDeviceTopic, getSubDevicesCb)
    checkDeviceStatus()
  })

  const route = useRoute()

  watch(() => route.path, (val) => {
    if (!['/station/data-transfer', '/station'].includes(val)) {
      clearInterval(checkDeviceStatusInterval.value)
      worker.unsubscribe(subDevices)
    }
  })



  function upFirmwareStatus(sn: string, feedback: any[]) {
    const device = devicesMap.get(sn)
    if (device) {
      device.upFirmware = 'updating'
      const step = feedback[1]
      const progress = feedback[2] || undefined
      const errMsg = feedback[3] || undefined
      if (step < (device.upFirmwareStatus?.step ?? -100)) return
      device.upFirmwareStatus = {
        step,
        progress: progress === -1 ? 100 : progress,
        errMsg,
      }
    }
  }

  function upFirmwareStatusReject(sn: string, feedback: any[]) {
    const device = devicesMap.get(sn)
    if (device) {
      device.upFirmware = 'rejected'
      const step = feedback[1]
      const progress = feedback[2] || undefined
      const errMsg = feedback[3] || undefined
      if (step < (device.upFirmwareStatus?.step ?? -100)) return
      device.upFirmwareStatus = {
        step,
        progress: progress === -1 ? 100 : progress,
        errMsg,
      }
    }
  }

  function upFirmwarePending(snList: string[]) {
    for (const sn of snList) {
      const device = devicesMap.get(sn)
      if (device) {
        device.upFirmware = 'pending'
        device.upFirmwareStatus = {
          step: 0,
          progress: undefined,
          errMsg: undefined,
        }
      }
    }
  }

  function upFirmwareReset(snList: string[]) {
    for (const sn of snList) {
      const device = devicesMap.get(sn)
      if (device) {
        upFirmwareSucceed(sn)
      }
    }
  }

  function upFirmwareSucceed(sn: string) {
    const device = devicesMap.get(sn)
    if (device) {
      device.upFirmware = undefined
      device.upFirmwareStatus = undefined
    }
  }

  function upFirmwareTimeout(sn: string) {
    const device = devicesMap.get(sn)
    if (device) {
      device.upFirmware = 'timeout'
      device.upFirmwareStatus = undefined
    }
  }


  return {
    isConnected,
    devicesMap,
    connectSite,
    checkDeviceStatusInterval,
    onlineCount,
    offlineCount,
    checkDeviceStatus,
    initConnectSite,
    upFirmwarePending,
    upFirmwareReset,
    upFirmwareStatus,
    upFirmwareSucceed,
    upFirmwareStatusReject,
    upFirmwareTimeout
  }
})
